import path from 'node:path';
import { compact } from '../../util/array.js';
import { _load } from '../../util/loader.js';
import { getPackageName } from '../../util/modules.js';
import { isAbsolute } from '../../util/path.js';
import { require } from '../../util/require.js';
const getDependencies = (config) => {
    const extend = config.extends ? [config.extends].flat().map(customResolvePluginPackageNames) : [];
    if (extend.includes('eslint-plugin-prettier'))
        extend.push('eslint-config-prettier');
    const plugins = config.plugins ? config.plugins.map(resolvePluginPackageName) : [];
    const parser = config.parser;
    const extraParsers = config.parserOptions?.babelOptions?.presets ?? [];
    const settings = config.settings ? getDependenciesFromSettings(config.settings) : [];
    return compact([...extend, ...plugins, parser, ...extraParsers, ...settings]).map(getPackageName);
};
export const getDependenciesDeep = async (configFilePath, dependencies = new Set()) => {
    const addAll = (deps) => deps.forEach(dependency => dependencies.add(dependency));
    const config = await _load(configFilePath);
    if (config.extends) {
        for (const extend of [config.extends].flat()) {
            if (extend.startsWith('.') || (isAbsolute(extend) && !extend.includes('/node_modules/'))) {
                const extendConfigFilePath = isAbsolute(extend)
                    ? extend
                    : require.resolve(path.join(path.dirname(configFilePath), extend));
                addAll(await getDependenciesDeep(extendConfigFilePath));
            }
        }
    }
    if (config.overrides)
        for (const override of config.overrides)
            addAll(getDependencies(override));
    addAll(getDependencies(config));
    return dependencies;
};
const resolvePackageName = (namespace, pluginName) => {
    return pluginName.startsWith('@')
        ? pluginName.includes('/')
            ? pluginName.replace(/\//, `/${namespace}-`)
            : `${pluginName}/${namespace}`
        : `${namespace}-${pluginName}`;
};
export const resolvePluginPackageName = (pluginName) => resolvePackageName('eslint-plugin', pluginName);
const customResolvePluginPackageNames = (extend) => {
    if (extend.includes('/node_modules/'))
        return getPackageName(extend);
    if (isAbsolute(extend) || extend.startsWith('.'))
        return;
    if (extend.includes(':')) {
        const pluginName = extend.replace(/^plugin:/, '').replace(/(\/|:).+$/, '');
        if (pluginName === 'eslint')
            return;
        return resolvePackageName('eslint-plugin', pluginName);
    }
    return extend.includes('eslint') ? getPackageName(extend) : resolvePackageName('eslint-config', extend);
};
const getImportPluginDependencies = (settings) => {
    const knownKeys = ['typescript'];
    if (Array.isArray(settings))
        return [];
    return Object.keys(settings)
        .filter(key => key !== 'node')
        .map(key => (knownKeys.includes(key) ? `eslint-import-resolver-${key}` : key));
};
export const getDependenciesFromSettings = (settings = {}) => {
    return compact(Object.entries(settings).reduce((packageNames, [settingKey, settings]) => {
        if (/^import\/(parsers|resolvers)?/.test(settingKey)) {
            return [...packageNames, ...getImportPluginDependencies(settings)];
        }
        return packageNames;
    }, []));
};
